""" A proxy for a workbench action. """


# Enthought library imports.
from enthought.pyface.api import ImageResource
from enthought.envisage import get_application
from enthought.traits.api import Any, Bool, Instance, Property, Str

# Local imports.
from selection_listener_action import SelectionListenerAction
from workbench_action import WorkbenchAction

from enthought.envisage.ui.ui_plugin_definition import DisabledWhen
from enthought.envisage.ui.ui_plugin_definition import EnabledWhen


class WorkbenchActionProxyFactory:

    def create_instance(self, action, window, extension_id):
        """ Creates a proxy action that will lazily load an implementation. """

        proxy = WorkbenchActionProxy(
            bundle = action,
            
            accelerator   = action.accelerator,
            checked       = action.checked,
            class_name    = action.class_name,
            defined_in    = extension_id,
            description   = action.description,
            disabled_when = action.disabled_when,
            enabled       = action.enabled,
            enabled_when  = action.enabled_when,
            image_path    = action.image,
            name          = action.name,
            style         = action.style,
            tooltip       = action.tooltip,
            window        = window,
        )

        return proxy

        
class WorkbenchActionProxy(SelectionListenerAction):
    """ A proxy for a workbench action.

    The proxy lazily loads the actual action implementation when the action
    is first performed.

    """

    #### 'Action' interface ###################################################

    # The action's image (displayed on tool bar tools etc).
    image = Property(Any)

    #### 'WorkbenchActionProxy' interface #####################################

    # The action that we are a proxy for (lazily loaded from the class name).
    action = Property(Instance(WorkbenchAction))

    # The name of the class that implements the action that we are a proxy
    # for.
    class_name = Str
    
    # The Id of the plugin that the action was defined in.
    defined_in = Any#Str

    # The path to the image (usually relative to a plugin's definition file).
    image_path = Str


    bundle = Any
    menu_bar_path = Str
    
    enabled_when = Instance(EnabledWhen)
    disabled_when = Instance(DisabledWhen)

    
    #### Private interface ####################################################
    
    # Shadow trait for the 'action' property.
    _action = Any

    # Shadow trait for the 'image' property.
    _image = Any

    _live = Bool(False)
    
    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __init__(self, **traits):
        """ Creates a new action proxy. """

        # Base class constructor.
        super(WorkbenchActionProxy, self).__init__(**traits)

        # Listen for changes to the checked state of the action (we can't use
        # a static trait handler here in case the checked state is set in the
        # call to this constructor.
        self.on_trait_change(self._on_checked_changed, 'checked')

        # fixme: This is a hack, but the intent is that if an action is for
        # ALL resource types then don't lazy load it... This is so that
        # actions like cop and paste etc can control their own enablement.
        if hasattr(self.bundle, 'resource_type'):
            if len(self.bundle.resource_type) == 0:
                self._live = True
        
        return
    
    ###########################################################################
    # 'Action' interface.
    ###########################################################################

    #### Properties ###########################################################
    
    # image
    def _get_image(self):
        """ Returns the action's label image. """

        if self._image is None:
            definition = get_application().plugin_definition_registry.get_definition(
                self.defined_in
            )

            # Create the action's image resource relative to that directory.
            self._image = ImageResource(
                name=self.image_path, search_path=[definition.location]
            )

        return self._image

    #### Methods ##############################################################
    
    def destroy(self):
        """ Called when the action is no longer required.

        By default this method does nothing, but this would be a great place to
        unhook trait listeners etc.

        """

        super(WorkbenchActionProxy, self).destroy()

        if self._action is not None:
            self._action.on_trait_change(
                self._on_enabled_changed, 'enabled', remove=True
            )
            self._action.destroy()
            
        return
    
    ###########################################################################
    # 'WorkbenchActionProxy' interface.
    ###########################################################################

    #### Properties ###########################################################
    
    # action
    def _get_action(self):
        """ Returns the actual action. """

        if self._action is None:
            
            if len(self.bundle.function_name) > 0:
                from function_action import FunctionAction
                self._action = FunctionAction(function_name=self.bundle.function_name)
                return self._action

            # We are about to actually import the action class, so make sure
            # that the plugin that contributed it has been started.
            get_application().plugin_activator.start_plugin(self.defined_in)
            
            # Import the implementation class
            klass = get_application().import_manager.import_symbol(self.class_name)

            # Create an instance of it.
            if len(self.bundle.id) > 0:
                self._action = klass(
                    name=self.name, style=self.style, window=self.window,
                    id = self.bundle.id
                )
            else:
                self._action = klass(
                    name=self.name, style=self.style, window=self.window
                )

            self._action.on_trait_change(self._on_enabled_changed, 'enabled')
            
            # We don't set this is in the constructor in case the action uses
            # a static trait handler to listen for changes to the 'checked'
            # trait (which is a common idiom).  If we did set it in the
            # constructor then it might fire before the name, style and window
            # traits had been initialized (since we can't guarantee the order
            # of traits initialization via a **kw argument).
            self._action.checked = self.checked
            
        return self._action

    ###########################################################################
    # 'Action' interface.
    ###########################################################################

    def perform(self, event):
        """ Performs the action. """                

        self.action.checked = self.checked

        import inspect

        # fixme: the 'perform' method without taking a context is deprecated!
        args, varargs, varkw, dflts = inspect.getargspec(self.action.perform)

        # If the only argument is 'self' then this is the DEPRECATED 
        # interface.
        if len(args) == 1:
            self.action.perform()
            
        else:
            self.action.perform(event)

        return

    ###########################################################################
    # 'SelectionListenerAction' interface.
    ###########################################################################

    def refresh(self):
        """ Refresh the enabled/disabled state of the action etc. """

        if self.window is not None:
            selection = self.window.selection

            if self._live:
                self.action.window = self.window
                self.enabled = self.action.enabled

            if self.enabled_when is not None:
                # Enabled when...
                if len(self.enabled_when.resource_type) > 0:
                    self._check_resource_type(
                        self.enabled_when.resource_type, selection
                    )

                elif len(self.enabled_when.cookie) > 0:
                    self._check_cookie(
                        self.enabled_when.cookie, selection
                    )

                elif len(self.enabled_when.parent_cookie) > 0:
                    self._check_parent_cookie(
                        self.enabled_when.parent_cookie, selection
                    )

                    
            if self.disabled_when is not None:
                # Disabled when...
                if len(self.disabled_when.resource_type) > 0:
                    self._check_disable_resource_type(
                        self.disabled_when.resource_type, selection
                    )

                elif len(self.disabled_when.cookie) > 0:
                    self._check_disable_cookie(
                        self.disabled_when.cookie, selection
                    )

                elif len(self.disabled_when.parent_cookie) > 0:
                    self._check_disable_parent_cookie(
                        self.disabled_when.parent_cookie, selection
                    )

        return
    
    ###########################################################################
    # Private interface.
    ###########################################################################
    
    #### Trait event handlers #################################################
    
    def _on_checked_changed(self):
        """ Called when the checked state of the action is changed. """

        self.action.checked = self.checked

        return

    def _on_enabled_changed(self):
        """ Called when the checked state of the action is changed. """

        self.enabled = self.action.enabled

        return

    def _check_resource_type(self, id, selection):

        rm = get_application().service_registry.get_service(
            'enthought.envisage.resource.IResourceManager'
        )
        rt = rm.lookup(id)

        for binding in selection:
            if rm.get_type_of(binding.obj) is not rt:
                self.enabled = False
                break

        else:
            self.enabled = True


        return

    def _check_cookie(self, id, selection):

        from enthought.envisage.core.import_manager import ImportManager
        im = ImportManager()
        klass = im.import_symbol(id)

        cm = get_application().service_registry.get_service(
            'enthought.envisage.resource.ICookieManager'
        )

        for binding in selection:
            cookie = cm.get_cookie(klass, binding.obj)
            if cookie is None:
                self.enabled = False
                break

        else:
            self.enabled = True

        return

    def _check_parent_cookie(self, id, selection):
        
        from enthought.envisage.core.import_manager import ImportManager
        im = ImportManager()
        klass = im.import_symbol(id)

        from enthought.envisage.project.cookie_manager import CookieManager
        cm = CookieManager()

        for binding in selection:
            if binding.context is None:
                self.enabled = False
                break

            cookie = cm.get_cookie(klass, binding.context)
            if cookie is None:
                self.enabled = False
                break

        else:
            self.enabled = True

        return

    def _check_disable_resource_type(self, id, selection):

        rm = get_application().service_registry.get_service(
            'enthought.envisage.resource.IResourceManager'
        )
        rt = rm.lookup(id)

        for binding in selection:
            if rm.get_type_of(binding.obj) is rt:
                self.enabled = False
                break

        else:
            self.enabled = True


        return

    def _check_disable_cookie(self, id, selection):

        from enthought.envisage.core.import_manager import ImportManager
        im = ImportManager()
        klass = im.import_symbol(id)

        from enthought.envisage.project.cookie_manager import CookieManager
        cm = CookieManager()

        for binding in selection:
            cookie = cm.get_cookie(klass, binding.obj)
            if cookie is not None:
                self.enabled = False
                break

        else:
            self.enabled = True

        return

    def _check_disable_parent_cookie(self, id, selection):
        
        from enthought.envisage.core.import_manager import ImportManager
        im = ImportManager()
        klass = im.import_symbol(id)

        from enthought.envisage.project.cookie_manager import CookieManager
        cm = CookieManager()

        for binding in selection:
            if binding.context is None:
                self.enabled = True
                break

            cookie = cm.get_cookie(klass, binding.context)
            if cookie is not None:
                self.enabled = False
                break

        else:
            self.enabled = True

        return
                
#### EOF ######################################################################
