""" A view containing an interactive Python shell. """


# Standard library imports.
import logging
import sys
from sets import Set

# Enthought library imports.
from enthought.envisage.workbench import View
from enthought.pyface.api import PythonShell
from enthought.traits.api import Property, Event, Any

# Plugin definition imports.
from enthought.plugins.python_shell.python_shell_plugin_definition import \
     Namespace


# Setup a logger for this module.
logger = logging.getLogger(__name__)


class TeeFile(object):
    """ Wrap a writable file to write to it and also call another function.
    """

    def __init__(self, fp, write_func):
        # The base file object.
        self.fp = fp

        # Another function to call when we write some text.
        self.write_func = write_func

    def readline(self):
        pass

    def write(self, text):
        self.fp.write(text)
        self.fp.flush()
        self.write_func(text)

    def writelines(self, lines):
        map(self.write, lines)

    def flush(self):
        self.fp.flush()

    def isatty(self):
        return self.fp.isatty()


class PythonShellView(View):
    """ A view containing an interactive Python shell. """

    #### 'PythonShellView' interface ##########################################

    # The interpreter's namespace.
    namespace = Property

    # The names bound in the interpreter's namespace.
    names = Property

    # Original value for 'sys.stdout':
    original_stdout = Any

    # Stdout text is posted to this event
    stdout_text = Event

    ###########################################################################
    # 'View' interface.
    ###########################################################################

    def create_control(self, parent):
        """ Creates the toolkit-specific control that represents the view.

        'parent' is the toolkit-specific control that is the view's parent.

        """

        # Push the replacement stdout before we instantiate PythonShell so it
        # will get this instead of the original one.
        self.original_stdout = sys.stdout
        sys.stdout = TeeFile(self.original_stdout, self._write_stdout)

        self.shell = shell = PythonShell(parent)
        shell.control.interp.locals['view'] = self
        shell.on_trait_change(self._on_key_pressed, 'key_pressed')
        shell.on_trait_change(self._on_command_executed, 'command_executed')

        # Write application standard out to this shell instead of to DOS window
        self.on_trait_change(self._on_write_stdout, 'stdout_text',
                             dispatch='ui')

        # Namespace contributions.
        logger.debug("Parsing namespace contributions to python shell view")
        extensions = self.application.get_extensions(Namespace)
        for extension in extensions:
            # Bindings.
            for name, value in extension.bindings.items():
                logger.debug("  binding ['%s', '%s']" % (name, value))
                self.bind(name, value)

            # Commands.
            for command in extension.commands:
                logger.debug("  command '%s'" % (command))
                self.execute_command(command)

        # We take note of the starting set of names bound in the interpreter's
        # namespace so that we can show the user what they have added or
        # removed in the namespace view.
        self._names = Set(self.names)

        # Register the view as a service.
        self.application.register_service(
            'enthought.plugins.IPythonShell', self
        )

        return self.shell.control

    def destroy_control(self):
        """ Destroys the toolkit-specific control that represents the view.
        """
        super(PythonShellView, self).destroy_control()

        # Remove the sys.stdout handlers.
        self.on_trait_change(self._on_write_stdout, 'stdout_text',
                             remove=True)
        sys.stdout = self.original_stdout

        return

    ###########################################################################
    # 'PythonShellView' interface.
    ###########################################################################

    #### Properties ###########################################################

    def _get_namespace(self):
        """ Property getter. """

        return self.shell.control.interp.locals

    def _get_names(self):
        """ Property getter. """

        return self.shell.control.interp.locals.keys()

    #### Methods ##############################################################

    def bind(self, name, value):
        """ Binds a name to a value in the interpreter's namespace. """

        self.shell.bind(name, value)

        return

    def execute_command(self, command, hidden=True):
        """ Execute a command in the interpreter. """

        return self.shell.execute_command(command, hidden)

    def lookup(self, name):
        """ Returns the value bound to a name in the interpreter's namespace.

        """

        return self.shell.control.interp.locals[name]

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _write_stdout(self, text):
        """ Handles text written to stdout. """
        self.stdout_text = text

    #### Trait change handlers ################################################

    def _on_write_stdout(self, text):
        self.shell.control.write(text)

    def _on_command_executed(self, shell):
        """ Dynamic trait change handler. """

        if self.control is not None:
            # Get the names that are now bound in the namespace.
            names = Set(self.names)

            # Find the differences in the namespace caused by the command
            # execution.
            added   = names.difference(self._names)
            removed = self._names.difference(names)

            # Remember the new state of the namespace.
            self._names = names

            # Trait event notification.
            if len(added) > 0 or len(removed) > 0:
                # fixme: We might want to get a tad more granular in the event
                # that we fire!
                self.trait_property_changed('namespace', {}, self.namespace)
                self.trait_property_changed('names', [], self.names)

        return

    def _on_key_pressed(self, event):
        """ Dynamic trait change handler. """

        if event.alt_down and event.key_code == 317:
            zoom = self.shell.control.GetZoom()
            if zoom != 20:
                self.shell.control.SetZoom(zoom+1)

        elif event.alt_down and event.key_code == 319:
            zoom = self.shell.control.GetZoom()
            if zoom != -10:
                self.shell.control.SetZoom(zoom-1)

        return

#### EOF ######################################################################
